@@ -324,18 +324,9 @@ static void resolve_addr(RES_CONTEXT *rp, char *sender, char *addr,
 	    tok822_free(tree->head);
 	    tree->head = 0;
 	}
-	/* XXX Re-resolve the surrogate, in case already in user@domain form. */
-	if (tree->head == 0) {
+	/* XXX must be localpart only, not user@domain form. */
+	if (tree->head == 0)
 	    tree->head = tok822_scan(var_empty_addr, &tree->tail);
-	    continue;
-	}
-
-	/* XXX Re-resolve with @$myhostname for backwards compatibility. */
-	if (domain == 0 && saved_domain == 0) {
-	    tok822_sub_append(tree, tok822_alloc('@', (char *) 0));
-	    tok822_sub_append(tree, tok822_scan(var_myhostname, (TOK822 **) 0));
-	    continue;
-	}
 
 	/*
 	 * We're done. There are no domains left to strip off the address,
@@ -462,7 +453,7 @@ static void resolve_addr(RES_CONTEXT *rp, char *sender, char *addr,
 #endif
 	    }
 	    vstring_strcpy(channel, MAIL_SERVICE_ERROR);
-	    vstring_sprintf(nexthop, "User unknown%s",
+	    vstring_sprintf(nexthop, "5.1.1 User unknown%s",
 			    var_show_unk_rcpt_table ?
 			    " in virtual alias table" : "");
 	    *flags |= RESOLVE_CLASS_ALIAS;
@@ -549,20 +540,15 @@ static void resolve_addr(RES_CONTEXT *rp, char *sender, char *addr,
 		if (*relay == 0) {
 		    msg_warn("%s: ignoring null lookup result for %s",
 			     rp->snd_relay_maps_name, sender_key);
-		    relay = 0;
-		} else if (strcasecmp(relay, "DUNNO") == 0)
-		    relay = 0;
+		    relay = "DUNNO";
+		}
+		vstring_strcpy(nexthop, strcasecmp(relay, "DUNNO") == 0 ?
+			       rcpt_domain : relay);
 	    } else if (rp->snd_relay_info
 		       && rp->snd_relay_info->error != 0) {
 		msg_warn("%s lookup failure", rp->snd_relay_maps_name);
 		*flags |= RESOLVE_FLAG_FAIL;
 		FREE_MEMORY_AND_RETURN;
-	    } else {
-		relay = 0;
-	    }
-	    /* Enforce all the relayhost precedences in one place. */
-	    if (relay != 0) {
-		vstring_strcpy(nexthop, relay);
 	    } else if (*RES_PARAM_VALUE(rp->relayhost))
 		vstring_strcpy(nexthop, RES_PARAM_VALUE(rp->relayhost));
 	    else
